@@ -18,6 +18,7 @@ describe('Core element tests', function() {
 		element.transition(0.25);
 
 		expect(element._view).toEqual(element._model);
+		expect(element._view).not.toBe(element._model);
 		expect(element._view.objectProp).toBe(element._model.objectProp); // not cloned
 
 		element._model.numberProp = 100;
@@ -40,5 +41,11 @@ describe('Core element tests', function() {
 			},
 			colorProp: 'rgb(64, 64, 0)',
 		});
+
+		// Final transition clones model into view
+		element.transition(1);
+
+		expect(element._view).toEqual(element._model);
+		expect(element._view).not.toBe(element._model);
 	});
 });
